# Generated by Django 4.2.5 on 2023-09-08 03:06

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('subjects', '0001_squashed_0009_delete_contactold'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='DataRevocationRequest',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='Created at')),
                ('type', models.PositiveSmallIntegerField(choices=[(0, 'All data'), (1, 'Contact data')], verbose_name='Type')),
                ('contact', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='subjects.contact', verbose_name='Contact')),
                ('creator', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to=settings.AUTH_USER_MODEL, verbose_name='Creator')),
            ],
            options={
                'verbose_name': 'Data revocation request',
                'verbose_name_plural': 'Data revocation requests',
            },
        ),
        migrations.CreateModel(
            name='DataAccessRequest',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='Created at')),
                ('contact', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='subjects.contact', verbose_name='Contact')),
                ('creator', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to=settings.AUTH_USER_MODEL, verbose_name='Creator')),
            ],
            options={
                'verbose_name': 'Data access request',
                'verbose_name_plural': 'Data access requests',
            },
        ),
    ]



from rest_framework import serializers

from huscy.attributes.services import get_or_create_attribute_set
from huscy.data_protection.models import DataAccessRequest, DataRevocationRequest
from huscy.subjects.models import Subject
from huscy.subjects.serializers import SubjectSerializer


class DataAccessRequestSerializer(serializers.ModelSerializer):
    creator = serializers.HiddenField(default=serializers.CurrentUserDefault())
    creator_name = serializers.CharField(source='creator.get_full_name', read_only=True)

    class Meta:
        model = DataAccessRequest
        fields = 'contact', 'created_at', 'creator', 'creator_name', 'id'
        read_only_fields = 'created_at',


class DataAccessSerializer(serializers.Serializer):
    attributes = serializers.SerializerMethodField()
    subject = serializers.SerializerMethodField()

    def get_attributes(self, contact):
        subject = Subject.objects.get(contact=contact)
        attribute_set = get_or_create_attribute_set(subject)
        return attribute_set.attributes

    def get_subject(self, contact):
        subject = Subject.objects.get(contact=contact)
        return SubjectSerializer(subject).data


class DataRevocationRequestSerializer(serializers.ModelSerializer):
    creator = serializers.HiddenField(default=serializers.CurrentUserDefault())
    creator_name = serializers.CharField(source='creator.get_full_name', read_only=True)

    class Meta:
        model = DataRevocationRequest
        fields = 'contact', 'created_at', 'creator', 'creator_name', 'id', 'type'
        read_only_fields = 'created_at',



from django.urls import include, path
from rest_framework.routers import DefaultRouter

from huscy.data_protection import views


router = DefaultRouter()
router.register('dataaccessrequests', views.DataAccessRequestViewSet)
router.register('datarevocationrequests', views.DataRevocationRequestViewSet)


urlpatterns = [
    path('api/', include(router.urls)),
]



import pytest

from rest_framework.reverse import reverse
from rest_framework.status import (HTTP_200_OK, HTTP_201_CREATED, HTTP_204_NO_CONTENT,
                                   HTTP_403_FORBIDDEN)

pytestmark = pytest.mark.django_db


def test_admin_user_can_apply_data_access_request(admin_client, data_access_request,
                                                  attribute_schema):
    response = apply_request(admin_client, data_access_request)

    assert response.status_code == HTTP_200_OK


def test_admin_user_can_create_data_access_request(admin_client, contact):
    response = create_request(admin_client, contact)

    assert response.status_code == HTTP_201_CREATED


def test_admin_user_can_delete_data_access_request(admin_client, data_access_request):
    response = delete_request(admin_client, data_access_request)

    assert response.status_code == HTTP_204_NO_CONTENT


def test_admin_user_can_list_data_access_requests(admin_client):
    response = list_requests(admin_client)

    assert response.status_code == HTTP_200_OK


def test_normal_user_cannot_apply_data_access_request(client, data_access_request,
                                                      attribute_schema):
    response = apply_request(client, data_access_request)

    assert response.status_code == HTTP_403_FORBIDDEN


def test_user_without_permissions_can_create_data_access_request(client, contact):
    response = create_request(client, contact)

    assert response.status_code == HTTP_201_CREATED


def test_user_without_permissions_cannot_delete_data_access_request(client, data_access_request):
    response = delete_request(client, data_access_request)

    assert response.status_code == HTTP_403_FORBIDDEN


def test_user_without_permissions_cannot_list_data_access_requests(client):
    response = list_requests(client)

    assert response.status_code == HTTP_403_FORBIDDEN


def test_anonymous_user_cannot_apply_data_access_request(anonymous_client, data_access_request,
                                                         attribute_schema):
    response = apply_request(anonymous_client, data_access_request)

    assert response.status_code == HTTP_403_FORBIDDEN


def test_anonymous_user_cannot_create_data_access_request(anonymous_client, contact):
    response = create_request(anonymous_client, contact)

    assert response.status_code == HTTP_403_FORBIDDEN


def test_anonymous_user_cannot_delete_data_access_request(anonymous_client, data_access_request):
    response = delete_request(anonymous_client, data_access_request)

    assert response.status_code == HTTP_403_FORBIDDEN


def test_anonymous_user_cannot_list_data_access_requests(anonymous_client):
    response = list_requests(anonymous_client)

    assert response.status_code == HTTP_403_FORBIDDEN


def apply_request(client, obj):
    return client.post(reverse('dataaccessrequest-apply', kwargs=dict(pk=obj.pk)))


def create_request(client, contact):
    return client.post(reverse('dataaccessrequest-list'), data=dict(contact=contact.pk))


def delete_request(client, obj):
    return client.delete(reverse('dataaccessrequest-detail', kwargs=dict(pk=obj.pk)))


def list_requests(client):
    return client.get(reverse('dataaccessrequest-list'))



import pytest

from rest_framework.reverse import reverse
from rest_framework.status import (HTTP_200_OK, HTTP_201_CREATED, HTTP_204_NO_CONTENT,
                                   HTTP_403_FORBIDDEN)

from huscy.data_protection.models import DataRevocationRequest

pytestmark = pytest.mark.django_db


def test_admin_user_can_apply_data_revocation_request(admin_client, data_revocation_request):
    response = apply_request(admin_client, data_revocation_request)

    assert response.status_code == HTTP_204_NO_CONTENT


def test_admin_user_can_create_data_revocation_request(admin_client, contact):
    response = create_request(admin_client, contact)

    assert response.status_code == HTTP_201_CREATED


def test_admin_user_can_delete_data_revocation_request(admin_client, data_revocation_request):
    response = delete_request(admin_client, data_revocation_request)

    assert response.status_code == HTTP_204_NO_CONTENT


def test_admin_user_can_list_data_revocation_requests(admin_client):
    response = list_requests(admin_client)

    assert response.status_code == HTTP_200_OK


def test_user_without_permissions_cannot_apply_data_revocation_request(client,
                                                                       data_revocation_request):
    response = apply_request(client, data_revocation_request)

    assert response.status_code == HTTP_403_FORBIDDEN


def test_user_without_permissions_can_create_data_revocation_request(client, contact):
    response = create_request(client, contact)

    assert response.status_code == HTTP_201_CREATED


def test_user_without_permissions_cannot_delete_data_revocation_request(client,
                                                                        data_revocation_request):
    response = delete_request(client, data_revocation_request)

    assert response.status_code == HTTP_403_FORBIDDEN


def test_user_without_permissions_cannot_list_data_revocation_requests(client):
    response = list_requests(client)

    assert response.status_code == HTTP_403_FORBIDDEN


def test_anonymous_user_cannot_apply_data_revocation_request(anonymous_client,
                                                             data_revocation_request):
    response = apply_request(anonymous_client, data_revocation_request)

    assert response.status_code == HTTP_403_FORBIDDEN


def test_anonymous_user_cannot_create_data_revocation_request(anonymous_client, contact):
    response = create_request(anonymous_client, contact)

    assert response.status_code == HTTP_403_FORBIDDEN


def test_anonymous_user_cannot_delete_data_revocation_request(anonymous_client,
                                                              data_revocation_request):
    response = delete_request(anonymous_client, data_revocation_request)

    assert response.status_code == HTTP_403_FORBIDDEN


def test_anonymous_user_cannot_list_data_revocation_requests(anonymous_client):
    response = list_requests(anonymous_client)

    assert response.status_code == HTTP_403_FORBIDDEN


def apply_request(client, obj):
    return client.post(reverse('datarevocationrequest-apply', kwargs=dict(pk=obj.pk)))


def create_request(client, contact):
    return client.post(
        reverse('datarevocationrequest-list'),
        data=dict(contact=contact.pk, type=DataRevocationRequest.TYPES.get_value('all_data'))
    )


def delete_request(client, obj):
    return client.delete(reverse('datarevocationrequest-detail', kwargs=dict(pk=obj.pk)))


def list_requests(client):
    return client.get(reverse('datarevocationrequest-list'))



import enum

from django.conf import settings
from django.db import models
from django.utils.translation import gettext_lazy as _

from huscy.subjects.models import Contact


class Request(models.Model):
    contact = models.ForeignKey(Contact, on_delete=models.CASCADE, verbose_name=_('Contact'))

    creator = models.ForeignKey(settings.AUTH_USER_MODEL, on_delete=models.PROTECT,
                                verbose_name=_('Creator'))
    created_at = models.DateTimeField(_('Created at'), auto_now_add=True)

    class Meta:
        abstract = True


class DataAccessRequest(Request):
    class Meta:
        verbose_name = _('Data access request')
        verbose_name_plural = _('Data access requests')


class DataRevocationRequest(Request):
    class TYPES(enum.Enum):
        all_data = (0, _('All data'))
        contact_data = (1, _('Contact data'))

        @classmethod
        def get_value(cls, member):
            return cls[member].value[0]

    type = models.PositiveSmallIntegerField(_('Type'), choices=[x.value for x in TYPES])

    class Meta:
        verbose_name = _('Data revocation request')
        verbose_name_plural = _('Data revocation requests')



from rest_framework import mixins, permissions, status, viewsets
from rest_framework.decorators import action
from rest_framework.response import Response

from huscy.data_protection.models import DataAccessRequest, DataRevocationRequest
from huscy.data_protection.serializer import (
    DataAccessRequestSerializer,
    DataAccessSerializer,
    DataRevocationRequestSerializer,
)


class RequestPermission(permissions.BasePermission):
    def has_permission(self, request, view):
        if request.method == 'POST':
            return True
        return request.user.is_superuser


class DataAccessRequestViewSet(mixins.CreateModelMixin, mixins.DestroyModelMixin,
                               mixins.ListModelMixin, viewsets.GenericViewSet):
    permission_classes = (permissions.IsAuthenticated, RequestPermission)
    queryset = DataAccessRequest.objects.all()
    serializer_class = DataAccessRequestSerializer

    @action(detail=True, methods=['POST'], permission_classes=(permissions.IsAdminUser, ))
    def apply(self, request, pk):
        data_access_request = self.get_object()
        serializer = DataAccessSerializer(instance=data_access_request.contact)
        return Response(data=serializer.data)


class DataRevocationRequestViewSet(mixins.CreateModelMixin, mixins.DestroyModelMixin,
                                   mixins.ListModelMixin, viewsets.GenericViewSet):
    permission_classes = (permissions.IsAuthenticated, RequestPermission)
    queryset = DataRevocationRequest.objects.all()
    serializer_class = DataRevocationRequestSerializer

    @action(detail=True, methods=['POST'], permission_classes=(permissions.IsAdminUser, ))
    def apply(self, request, pk):
        data_revocation_request = self.get_object()

        if data_revocation_request.type == DataRevocationRequest.TYPES.get_value('all_data'):
            # TODO: delete attributes, participation requests and participations
            pass

        # TODO: check, if contact is only guardian for a child. then, child must be removed first
        # or needs a new guardian
        data_revocation_request.contact.delete()

        return Response(status=status.HTTP_204_NO_CONTENT)
